# /etc/sysconfig/adclient may contain configurable
# environment variables:
#   ADCLIENT  Alternate daemon location and name
#   ADINFO    Alternate location for adinfo command
#   OPTIONS   Daemon command line options. These are combined with any
#             additional command line parameters.

if [ -f /etc/sysconfig/adclient ]; then
    . /etc/sysconfig/adclient
fi

ADINFO=${ADINFO:-/usr/bin/adinfo}
PIDFILE=${PIDFILE:-/var/run/adclient.pid}
DAEMON=${ADCLIENT:-/usr/sbin/adclient}

NAME="Centrify DirectControl"

# Set up a default search paths
LD_LIBRARY_PATH=/usr/share/centrifydc/lib:/usr/share/centrifydc/kerberos/lib:$LD_LIBRARY_PATH

GREP=${GREP:-/bin/grep}
PS=${PS:-/bin/ps}

NOECHO=${NOECHO:-0}

RC_OK=0
RC_FAIL=1

wait_adclient () {
        # wait for a maximun of 60 seconds for adclient to be connected
        MODE=`$ADINFO -m`
        MAX=60
        while [ $MAX -gt 0 ] && [ "$MODE" != "connected" ]; do
            MAX=`expr $MAX - 2`
            sleep 2
            MODE=`$ADINFO -m`
        done
        echo "adclient state is: $MODE"
}


adclient_pid () {
    if [ -f "$PIDFILE" ]; then
        PID=`cat $PIDFILE 2> /dev/null`
    else
        if PSOUT=`$PS -ef | $GREP -w adclient | $GREP -w 1 | $GREP -v grep`
            then
            PARENTPID=`echo $PSOUT | awk '{print $3}'`
            if [ "$PARENTPID" = 1 ]; then
                PID=`echo $PSOUT | awk '{print $2}'`
            fi
        fi
    fi
}

adclient_status () {
        adclient_pid
        # No PID: daemon not running
        if [ -z "$PID" ]; then
            [ $NOECHO -ne 1 ] && echo "$NAME is stopped"
            return $RC_FAIL
        fi

        # Verify PID
        if $PS -ef | $GREP -w $PID | $GREP -v "grep" > /dev/null; then
            [ $NOECHO -ne 1 ] && echo "$NAME (pid $PID) is running..."
            return $RC_OK
        else
            [ $NOECHO -ne 1 ] && echo "$NAME not running but PID file exists"
            return $RC_FAIL
        fi
}

adclient_check () {
        $ADINFO --zone > /dev/null
        if [ $? -ne 0 ]; then
            echo
            echo "  Failed: machine is not joined."
            exit 2
        fi

        NOECHO=1
        adclient_status
        if [ $? -eq 0 ]; then
            echo
            echo "daemon is already running (pid $PID)."
            exit 0
        fi
        NOECHO=0

        if [ ! -x $DAEMON ]; then
            echo
            echo "  Failed: daemon '$DAEMON' is not executable."
            exit 2
        fi
}
