#!/bin/sh
# Date: 03/9/2006
#
# samba-centrify-config.sh
#
# Script to automate the setup of Centrify DirectControl for Samba
#
# Copyright (C) 2005-2006 Centrify Corp.
# All rights reserved.
#

clear
OS=`uname`
ADINFO=/usr/bin/adinfo
ADSAMBA=/usr/share/centrifydc/sbin/adsamba

JOIN_CDC_DOMAIN="N"
ORG_CDC_DOMAIN=
CDC_DOMAIN=
CDC_COMPUTER=`${ADSAMBA} -H`
SHORTNAME=
ORG_CDC_ZONE=
CDC_ZONE=
CDC_ALIASES=
CDC_OPTIONS=
CDC_CFG_FILE=/etc/centrifydc/centrifydc.conf
CDC_DN=
CDC_PDC=
PASSWORD=
TARGET_OS=""
PCA_DIR=""

SMB_OPTIONS="-c -f "
SMB_CONF_MSG="Your existing SMB.CONF file will be backed up and replaced"

ECHO=/bin/echo
PROMPT=
DEFAULT=

case "$OS" in 
    "Linux")
        ECHO="/bin/echo"
        PROMPT="$ECHO -n "
        NNL=""
        INIT_SCRIPT_DIR=/etc/init.d
        RC_SCRIPT_DIR=/etc/rc.d
    ;;
    "SunOS")
        ECHO=/usr/ucb/echo
        PROMPT="$ECHO -n "
        TARGET_OS="solaris"
        NNL=""
        INIT_SCRIPT_DIR=/etc/init.d
        RC_SCRIPT_DIR=/etc
    ;;
    "HP-UX")
        ECHO="/usr/bin/echo"
        PROMPT="$ECHO"
        NNL="\c"
        INIT_SCRIPT_DIR=/sbin/init.d
        RC_SCRIPT_DIR=/sbin
    ;;
    "AIX")
        ECHO="/bin/echo"
        PROMPT="$ECHO"
        NNL="\c"
        INIT_SCRIPT_DIR=/etc/rc.d/init.d
        RC_SCRIPT_DIR=/etc
    ;;
    "IRIX64")
        ECHO="/sbin/echo"
        PROMPT="$ECHO -n "
        NNL=""
        INIT_SCRIPT_DIR=/etc/init.d
        RC_SCRIPT_DIR=/etc
    ;;
    *)
        echo "Unknown Operating System $OS! "
        exit
    ;;
esac

# some script defines
SAMBA_SEARCH_DIRS=`awk '$1=="samba.base.path:" {print $2}' ${CDC_CFG_FILE} 2>/dev/null` 
: ${SAMBA_SEARCH_DIRS:=/opt/centrify/samba}

# smb.conf provided by blessed samba
CDC_SMBCONF=

# the number of smb.conf 
SMBCNT=-1
#
# Print an error and exit.
#
error()
{
    $ECHO "ERROR: $*" >&2
    exit 1
}

#
# Check that this script is being run by a superuser (root)
#
uid_check()
{
    id | grep uid=0 > /dev/null
    if [ "$?" != "0" ]; then
        error "Only the superuser can run this script."
    fi
}

#
# Find the Samba programs
#
find_samba()
{
	
    for dir in $SAMBA_SEARCH_DIRS
    do
        
        if [ -x "$dir/bin/smbstatus" -a ! -d "$dir/bin/smbstatus" ]
        then
            CDC_SAMBA_ROOT="$dir"
            CDC_SMBCONF="`$dir/sbin/smbd -b | awk '/CONFIGFILE: /{print $2}'`"
            SMBCNT=`expr ${SMBCNT} + 1`
            nEXISTING_SMBCONF_ARR=EXISTING_SMBCONF_ARR${SMBCNT}
            eval "${nEXISTING_SMBCONF_ARR}=\"$CDC_SMBCONF\""
            return 0
        fi
    done

    error "Cannot find Samba. Aborting script."
}

### Ask the QUESTION with possible DEFAULT value and read the ANSWER (accept empty input)
do_ask() 
{
    if [ "$DEFAULT" != "" ]; then
        $PROMPT "${QUESTION} [$DEFAULT] :" $NNL
    else
        $PROMPT "${QUESTION} :" $NNL
    fi

    ANSWER=""; read ANSWER

    if [ "${ANSWER}" = ""  -a  "${DEFAULT}" != "" ]; then
        ANSWER=$DEFAULT
    fi

    if [ "${ANSWER}" = "q" -o "${ANSWER}" = "Q" ]; then do_quit; fi
}

### Ask YES or NO QUESTION and read the ANSWER
do_askyorn() 
{
    ANSWER=""
    while [ "${ANSWER}" != "Y" -a "${ANSWER}" != "y" -a "${ANSWER}" != "N" -a "${ANSWER}" != "n" ]
    do
        do_ask
        if [ "${ANSWER}" = "" ]; then ANSWER="Y"; fi
    done

    DEFAULT=""
    ANSWER=`$ECHO "${ANSWER}" | tr "[a-z]" "[A-Z]"` 
}

### Ask for a required ANSWER (DEFAULT can be supplied)
ask_required() 
{
    ANSWER=""
    while [ "${ANSWER}" = "" ]; do
        do_ask
        
        if [ "${ANSWER}" = "" ]; then
            $ECHO "You must enter a value or press Q to quit"
        fi
    done

    DEFAULT=""
    $ECHO "Using ($ANSWER)"
}

### Exit by user request
do_quit () 
{
    $ECHO Installation terminated. Exiting ...
    exit 2
}

#
# Find the Samba programs
#
find_conflicting_samba()
{
    # Looking if there's any conflicting samba installed .
    # ignored any line contain symbolic link information
    FOUND_SMBD_LIST=`${ADSAMBA} -o -V 2>&1 | awk -F': ' '/Found: |Symlink Target: /{ if (! ($2 ~ /symbolic/))  print $2}'`
    SMBD_COUNT=0;
    
    if [ "$FOUND_SMBD_LIST" != "" ]; then
        # Get the list of smb.conf, insert to the list if the file exist
        # Note: the first item in the array is $CDC_SMBCONF, let add it the next of the list
        
        for daemon in $FOUND_SMBD_LIST 
        do
            # Check if this smbd same as CDC version
            [ "${daemon}" = "${CDC_SAMBA_ROOT}/sbin/smbd" ] && continue;
        
            # increment the conflicting samba found
            SMBD_COUNT=`expr $SMBD_COUNT + 1`
                        
            # Let's get the existing smb.conf from the conflicting samba

            if [ -x "${daemon}" ]; then
                SMBCNT=`expr ${SMBCNT} + 1`
                nEXISTING_SMBCONF_ARR=EXISTING_SMBCONF_ARR${SMBCNT} 
                smb_conf=`${daemon} -b | awk '/CONFIGFILE: /{print $2}'`
                
                [ -f "$smb_conf" -a "$smb_conf" != "$CDC_SMBCONF" ] && eval "${nEXISTING_SMBCONF_ARR}=\"$smb_conf\""
                if [ $? -ne 0 ]; then
                	SMBCNT=`expr ${SMBCNT} - 1`
                fi
            fi
        done

        if [ "$SMBD_COUNT" -gt 0 ] 
        then
            $ECHO
            $ECHO "WARNING: One or more potentially conflicting Samba installations were found."
            $ECHO "We strongly recommend you remove them before configure Samba for Centrify DirectControl."
            $ECHO
            QUESTION="Do you want to continue?"; DEFAULT="N"; do_askyorn

            [ ${ANSWER} = "N" ] && do_quit 
            
        fi
    fi
}

###Checking the smb.conf file. 
#function: 
#         1.Checking if the security is ADS 
#         2.Checking if the workgroup is set 
#         3.Checking if the realm is set 
#return: 
#       0   success 
#       1   failure 
   
check_smbconf() 
{ 
    #$1:smb.conf path 
    SMBCONF=$1 
     
    #STATE=0,1.0 is start state,1 is transaction state    
    STATE=0 
     
    SECURIT= 
    WORKGROUP= 
    REALM= 
    RETN=0 
 
    if [ "X${SMBCONF}" != "X" -a  -s ${SMBCONF} ]; then 
    OLD_IFS=$IFS
    IFS='$'
        for LINE in `cat -v -e ${SMBCONF} | tr -d '\n'` 
        do 
            LINE=`echo $LINE | sed 's/^[ ]*//g'`
            if [ "X${LINE}" != "X" ]; then 
                case ${LINE} in  
                    \#*|\;*)  
                        continue 
                        ;; 
                    \[global\])  
                        #start deal with the global section 
                        STATE=1 
                        ;; 
                    \[*\]) 
                        #end deal with the global section 
                        STATE=0 
                        break 
                        ;; 
                    *) 
                        if [ "${STATE}" = "1" ]; then 
                            case ${LINE} in 
                                security*) 
                                    SECURIT="`$ECHO "${LINE}"|awk -F= '{print $2}'|sed 's/^[ ][ ]*//g'`" 
                                    ;; 
                                workgroup*) 
                                    WORKGROUP="`$ECHO "${LINE}"|awk -F= '{print $2}'|sed 's/^[ ][ ]*//g'`" 
                                    ;; 
                                realm*) 
                                    REALM="`$ECHO "${LINE}"|awk -F= '{print $2}'|sed 's/^[ ][ ]*//g'`" 
                                    ;; 
                            esac 
                        fi 
                        ;;  
                esac     
            fi 
        done 
        IFS=$OLD_IFS
    else 
        $ECHO "The smb.conf does not exist or is invalid." 
    fi 
     
    #check security 
    if [ "X${SECURIT}" != "XADS" ]; then 
        $ECHO 
        $ECHO "Value of security is not set to \"ADS\".File ${SMBCONF} must be updated, otherwise Samba will failed to interop with CDC." 
        RETN=1 
    fi 
     
    #check workgroup 
    if [ "X${WORKGROUP}" = "X" ]; then 
        $ECHO 
        $ECHO "Value of workgroup is not set.File ${SMBCONF} must be updated, otherwise Samba will failed to interop with CDC."
        RETN=1 
    fi 
     
    #check workgroup 
    if [ "X${REALM}" = "X" ]; then 
        $ECHO 
        $ECHO "Value of realm is not set.File ${SMBCONF} must be updated, otherwise Samba will failed to interop with CDC." 
        RETN=1 
    fi 
    $ECHO 
    return ${RETN} 
} 
   
#
# Let user select which smb.conf to use if there are more than one
# version of conflicting samba found
#
find_smb_conf()
{
    RETN=0 
    if [ "${SMBCNT}" -gt 0 ]
    then
        SELECT_SMBCONF=

        while [ "X$SELECT_SMBCONF" = "X" ]
        do
            i=0
            # Displayed the smb.conf for selection... 
            $ECHO
            $ECHO "Select the SMB.CONF that you would like to use:"
                        
            while [ ${i} -le ${SMBCNT} ]
            do
                smb_conf=EXISTING_SMBCONF_ARR${i}
                eval "$ECHO  [$i] \$${smb_conf}"
                i=`expr $i + 1`
            done

            # Get User selection
            QUESTION="Select smb.conf from above list "; DEFAULT=0; do_ask

            smb_conf_tmp=EXISTING_SMBCONF_ARR${ANSWER}
            SELECT_SMBCONF_TMP=SELECT_SMBCONF
            eval "${SELECT_SMBCONF_TMP}=\$${smb_conf_tmp}"
            
            if [ "X$SELECT_SMBCONF" = "X" ] 
            then
                $ECHO "You must enter the number listed above or press Q to exit."
            fi
        done

        # back up then overwrite the CDC smb.conf with selected version
        if [ "$SELECT_SMBCONF" != "$CDC_SMBCONF" ]; then
            $ECHO "${CDC_SMBCONF} will be replaced with ${SELECT_SMBCONF}."
            $ECHO 
            [ ! -f "$CDC_SMBCONF.pre_adsamba" ] && cp $CDC_SMBCONF $CDC_SMBCONF.pre_adsamba
            cp $SELECT_SMBCONF $CDC_SMBCONF
        fi
        $ECHO "Using $SELECT_SMBCONF"
   # when SMBCNT=0 indicate that there's only one cdc smb.conf in the the machine, however, 
   # we need to make sure this smb.conf really exists in the machine. If this file does not exist, 
   # ADSAMBA will generate a clean one
    elif [ ! -f ${CDC_SMBCONF} ]; then
        RETN=1
    fi
    return ${RETN}
}

### Ask user for information require to join or rejoin domain 
get_join_info()
{
    # Get domain
    QUESTION="Enter the Active Directory domain to join"; DEFAULT=${CDC_DOMAIN}; ask_required 
    CDC_DOMAIN=${ANSWER}

    # Check that DNS is set up correctly for the selected domain
    if [ "`$ADINFO -V --diag ${CDC_DOMAIN} 2>&1|grep 'DNS may be misconfigured'`" ]
    then
        $ECHO "    DNS appears to be misconfigured.  DNS must point to a DNS server that can"
        $ECHO "    resolve domain information for the ${CDC_DOMAIN} Active Directory domain."
        $ECHO "    Run adinfo --diag and check your /etc/nsswitch.conf or /etc/resolv.conf file."
        $ECHO "    See the DirectControl documentation for more information on DNS requirements."
        error "Exiting script."
    fi

    # Get zone
    if [ "X${ORG_CDC_DOMAIN}" != "X" ]; then
        # If new domain that we going to join is different from the original domain, set CDC_ZONE to default.
        if [ "X${ORG_CDC_DOMAIN}" != "X${CDC_DOMAIN}" ]; then
            CDC_ZONE=default
        fi
    fi
    QUESTION="Enter the name of the zone"; DEFAULT=${CDC_ZONE}; ask_required; CDC_ZONE=${ANSWER}

    # Get COMPUTER name to join as
    QUESTION="Enter the computer name"; DEFAULT=${CDC_COMPUTER}; ask_required; CDC_COMPUTER=${ANSWER}

    # Generate any aliases we can automatically detect
    gen_aliases

    # Get user credentials to join the domain
    QUESTION="Enter the Active Directory authorized user"; DEFAULT="Administrator"; ask_required; CDC_ADMIN=${ANSWER}

    # Get Administrator password
    QUESTION="Enter the password for the Active Directory user ${CDC_ADMIN}";get_password;CDC_PASSWORD="${PASSWORD}"
    ${ECHO}

    # Ask the user if they wish to supply some of the more esoteric options
    additional_options
    confirm_join_settings
    
}

do_join()
{
    #adleave currently joined Active Directory domain
    ADMIN=${CDC_ADMIN}
    ADMIN_PASSWORD=${CDC_PASSWORD}
    if [ "X${ORG_CDC_DOMAIN}" != "X" ]; then
        if [ "X${ORG_CDC_DOMAIN}" != "X${CDC_DOMAIN}" ]; then
            #The orginal domain does not the same as the new domain
            $ECHO "The domain that you going to join is different from the domain you has joined."
            $ECHO "Please enter username and password of authorized user to leave ${ORG_CDC_DOMAIN}."

            # Get user credentials to leave the domain
            QUESTION="Enter the Active Directory authorized user to leave ${ORG_CDC_DOMAIN}"; DEFAULT="Administrator"; ask_required; ADMIN=${ANSWER}
            # Get Administrator password
            QUESTION="Enter the password for the Active Directory user ${ADMIN} to leave ${ORG_CDC_DOMAIN}";get_password;ADMIN_PASSWORD="${PASSWORD}"
            ${ECHO}
        fi

        #Adleave and Check the return value of adleave command    
        adleave --user "${ADMIN}" --password "${ADMIN_PASSWORD}" > /dev/null
        retcode=$?
        if [ $retcode -ne "0" ]; then
            QUESTION="Do you want to force local to leave the domain (no network activity)?"; DEFAULT="N"; do_askyorn
            if [ "X${ANSWER}" = "XY" ]; then
                #Force local to leave ${ORG_CDC_DOMAIN}
                $ECHO "Force local to leave ${ORG_CDC_DOMAIN}."
                adleave --force
                retcode=$?
                if [ $retcode -ne "0" ]; then do_quit; fi  
            else
                do_quit    
            fi
        
        fi    
    fi

    # Remove krb5.keytab after adleave
    rm -f /etc/krb5/krb5.keytab
    rm -f /etc/krb5.keytab
    rm -f /etc/krb5.ccache

    ${ECHO}
    ${ECHO} "If prompted for a machine password, just press ENTER."
    ${ECHO} "Joining Active Directory domain with DirectControl and Samba"
    #adjoin to given Active Directory domain
    if [ "X${CDC_PDC}"  = "X" -a "X${CDC_DN}" != "X" ]; then
        #Computer Container
        adjoin "${CDC_DOMAIN}" -z "${CDC_ZONE}" -u "${CDC_ADMIN}" -p "${CDC_PASSWORD}" -n "${CDC_COMPUTER}" ${CDC_ALIASES} ${CDC_OPTIONS} --force -c "${CDC_DN}"
    elif [ "X${CDC_PDC}" != "X" -a "X${CDC_DN}"  = "X" ]; then
        #Preferred Domain 
        adjoin "${CDC_DOMAIN}" -z "${CDC_ZONE}" -u "${CDC_ADMIN}" -p "${CDC_PASSWORD}" -n "${CDC_COMPUTER}" ${CDC_ALIASES} ${CDC_OPTIONS} --force -s "${CDC_PDC}"
    elif [ "X${CDC_PDC}" != "X" -a "X${CDC_DN}" != "X" ]; then
        #Preferred Domain and Computer Container
        adjoin "${CDC_DOMAIN}" -z "${CDC_ZONE}" -u "${CDC_ADMIN}" -p "${CDC_PASSWORD}" -n "${CDC_COMPUTER}" ${CDC_ALIASES} ${CDC_OPTIONS} --force -c "${CDC_DN}" -s "${CDC_PDC}"
    else 
        adjoin "${CDC_DOMAIN}" -z "${CDC_ZONE}" -u "${CDC_ADMIN}" -p "${CDC_PASSWORD}" -n "${CDC_COMPUTER}" ${CDC_ALIASES} ${CDC_OPTIONS} --force 
    fi
    
    retcode=$?
    if [ $retcode  -ne "0" ]; then
        do_quit
    fi
}

### Check if the AD Domain name is different the DNS domain name. If so generate aliases
gen_aliases()
{
    AD_DOMAIN=`$ECHO "${CDC_DOMAIN}" | tr "[a-z]" "[A-Z]"`
    DNS_DOMAIN=`${ADSAMBA} -D | tr "[a-z]" "[A-Z]"`
    SHORTNAME=`echo "${CDC_COMPUTER}" |  cut -d "." -f 1`
    NAME_SUFFIX=`echo "${CDC_COMPUTER}" |  cut -d "." -f 2- | tr "[a-z]" "[A-Z]"` 

    if [ "x$DNS_DOMAIN" != "x" ] && [ "${AD_DOMAIN}" != "$DNS_DOMAIN" ] && [ "${DNS_DOMAIN}" != "${NAME_SUFFIX}" ]; then 
       dns_diff=1
    fi
    if  [ "${SHORTNAME}" != "${CDC_COMPUTER}" ] && [ "${AD_DOMAIN}" != "${NAME_SUFFIX}" ]; then
       host_diff=1
    fi 
    if [ "$host_diff" != "1" ] && [ "$dns_diff" != "1" ]; then
      # no additional aliases need tobe generated 
      return
    fi
    $ECHO "Your AD domain name is: ${AD_DOMAIN}"
    if [ "$dns_diff" = "1" ]; then
      $ECHO "Your DNS domain name is: ${DNS_DOMAIN}"
      alias1=${SHORTNAME}.${DNS_DOMAIN}
      CDC_ALIASES="-a ${alias1}"
    fi
    if [ "$host_diff" = "1" ]; then
      $ECHO "This machine will join as: ${CDC_COMPUTER}" 
      alias2=${SHORTNAME}.${AD_DOMAIN}
      CDC_ALIASES="${CDC_ALIASES} -a ${alias2}"
    fi
    $ECHO "Adding the alias(es) to cover this case(es):${alias1} ${alias2}" 

}

### Ask the user if they wish to add any additional options
additional_options()
{
    QUESTION="Do you wish to specify advanced join options?"; DEFAULT="N"; do_askyorn
    if [ "${ANSWER}" = "Y" ]; then
                     
        QUESTION="Active Directory Computer Container (press Enter for default)"; DEFAULT=""; do_ask
        if [ "${ANSWER}" != "" ]; then
            #-c option
            CDC_DN="${ANSWER}"
        fi

        QUESTION="Preferred Domain Server to use (press Enter for none)"; DEFAULT=""; do_ask
        if [ "${ANSWER}" != "" ]; then
            #-s option
            CDC_PDC="${ANSWER}"
        fi

        QUESTION="Update PAM and NSS Config?"; DEFAULT="Y"; do_askyorn
        if [ "${ANSWER}" = "N" ]; then
            CDC_OPTIONS="${CDC_OPTIONS} -C"
        fi

        QUESTION="Trust computer for delegation?"; DEFAULT="N"; do_askyorn
        if [ "${ANSWER}" = "Y" ]; then
            CDC_OPTIONS="${CDC_OPTIONS} -T"
        fi

        QUESTION="Use DES encryption only?"; DEFAULT="N"; do_askyorn
        if [ "${ANSWER}" = "Y" ]; then
            CDC_OPTIONS="${CDC_OPTIONS} -k"
        fi

        QUESTION="Run adjoin in verbose mode"; DEFAULT="N"; do_askyorn
        if [ "${ANSWER}" = "Y" ]; then
            CDC_OPTIONS="${CDC_OPTIONS} -V"
            SMB_OPTIONS="${SMB_OPTIONS} -V"
        fi

        $ECHO "Enter one alias each time you are prompted. Press Enter on a blank line to stop."

        ANSWER="some"
        while [ "${ANSWER}" != "" ]
        do
            QUESTION="Additional computer alias"; DEFAULT=""; do_ask
            if [ "${ANSWER}" != "" ]; then
                CDC_ALIASES="${CDC_ALIASES} -a ${ANSWER}"
            fi
        done
    fi
}

confirm_join_settings()
{
    $ECHO
    $ECHO
    $ECHO "The following settings will be used to join your computer to the AD Domain"
    $ECHO "The Active Directory domain is:" ${CDC_DOMAIN}
    $ECHO "Your computer will be joined as:" ${CDC_COMPUTER}.${CDC_DOMAIN}
    $ECHO "The Centrify Zone is:" ${CDC_ZONE}

    if [ "${CDC_ALIASES}" != "" ]; then
        $ECHO "Your computer aliases are: " ${CDC_ALIASES}
    fi
    
    TMPECHO=
    
    [ "X${CDC_DN}"      != "X" ] && TMPECHO=" -c ${CDC_DN}"
    [ "X${CDC_PDC}"     != "X" ] && TMPECHO="${TMPECHO} -s ${CDC_PDC}"
    [ "X${CDC_OPTIONS}" != "X" ] && TMPECHO="${TMPECHO} ${CDC_OPTIONS}"              
    [ "X${TMPECHO}"     != "X" ] && $ECHO "Your advanced options are: " ${TMPECHO}        

    $ECHO "The AD user for joining the domain is:" ${CDC_ADMIN}

    $ECHO "Samba `${CDC_SAMBA_ROOT}/bin/smbstatus -V` user programs were found in"
    $ECHO "${CDC_SAMBA_ROOT}/bin."
    $ECHO

    $ECHO "${SMB_CONF_MSG}"
    $ECHO

    QUESTION="Do you wish to proceed with the join?"; DEFAULT="Y"; do_askyorn
    if [ "${ANSWER}" = "N" ]; then
        do_quit
    fi
}

#
# Functions for getting the password
#
get_password()
{
    $PROMPT "${QUESTION} :" $NNL
    
    PASSWORD=""
    
    stty -echo
    trap "stty echo ; echo 'Interrupted' ; exit 1" 1 2 3 15
    read PASSWORD
    stty echo
}
### find Perl --need perl to run pca
is_perl_installed () 
{
    perl=
    $ECHO "looking for Perl ..." 
    PERL_DIRS="/bin /usr/bin /usr/local/bin /opt/bin /opt/perl/bin /opt/perl5/bin"
    for dir in `(IFS=:; set -- $PATH; echo "$@")` ${PERL_DIRS}
    do
        # If this file is a plain executable, we found it.
        if [ -x "$dir/perl" -a -f "$dir/perl" ]; then
            # Some systems have a broken perl earlier in the path;
            # test this one to make sure it works.
            if "$dir/perl" -e 'use strict;' > /dev/null 2>&1; then
                perl="$dir/perl"
                $ECHO "found Perl: ${perl}" 
                break
	    fi
        fi
    done

    if [ "$perl" = "" ]; then  
        $ECHO "WARNING:"       
        $ECHO "Could not find Perl required by PCA functionality."
        return 1
    fi
    return 0
}
### find pca script
search_pca ()
{
    $ECHO "search_pca ..."
    if [ "${TARGET_OS}" = "solaris" ]; then
        is_perl_installed || { return 1 ;}
        for dir in . ./pca ../SupportTools/pca ../AdditionalTools/pca
        do
            if [ -f "$dir/pca" -a -f "$dir/samba.xref" ]; then
                PCA_DIR=$dir
                $ECHO "Found pca script in ${PCA_DIR}"
                if [ ! -x "$dir/pca" ]; then
                    chmod 555 $dir/pca > /dev/null 2>&1
                fi
                return 0
            fi
        done
        if [ "${PCA_DIR}" = "" ]; then
           $ECHO "WARNING: Could not find Patch Check (PCA) script." 
           $ECHO "If you want to check this computer for missing required patches, please copy all"
           $ECHO "PCA files (or pca directory) next to the adsamba.sh and rerun this shell script." 
           QUESTION="Do you want to continue? [Y]"; do_askyorn
           if [ ${ANSWER} != "Y" ]; then do_quit; fi
        fi
    fi
    return 1
}

#
# Functions for commands not available in specific platforms
#

# pkill not available in aix, darwin, hpux, solaris 2.6
PKILL=`which pkill 2>/dev/null` 
if [ "x$PKILL" = "x" ]; then
    PKILL="/sbin/pkill"
fi

if [ ! -x "$PKILL" ]; then
    # syntax: pkill [-sig] processname
    pkill() {
        if [ $# -lt 1 ]; then
            print "Missing argument: process name"
            return $FAILURE
        fi
        if [ $# -eq 2 ]; then
            sig=$1
            shift
        fi
        case "$OS" in
        "Darwin")
            for pid in `ps -ax -opid,ucomm | fgrep $1 | awk '{print $1}'`; do
                kill $sig $pid
            done
            ;;
        *)
            for pid in `ps -ae | fgrep $1 | awk '{print $1}'`; do
                kill $sig $pid
            done
            ;;
        esac
    }
fi

#
# Begin main script
#

# Make sure its the Super user running
uid_check

# provide warnings
$ECHO "Before proceeding with this install, please make sure that you have"
$ECHO "read the Centrify Application Note for Samba"
$ECHO
$ECHO "Press ENTER to continue or CONTROL-C to exit ... "
read i
# Check to see if DirectControl is installed 
if [ ! -x "$ADINFO" -o -d "$ADINFO" ] ; then
    error "DirectControl does not seem to be installed. Exiting script"
fi

if [ "${TARGET_OS}" = "solaris" ]; then
   search_pca
   retcode=$?
   if [ $retcode -eq "0" ]; then
      if [ "${PCA_DIR}" != "" ]; then
         cd ${PCA_DIR}
         PCA_DIR=`pwd`
         if [ -d '/tmp' -a -w '/tmp' ]; then
             TMP='/tmp'
         else
             TMP='/var/tmp'
         fi
         rm -f ${TMP}/patchdiag.xref 2> /dev/null
         ln -s ${PCA_DIR}/samba.xref ${TMP}/patchdiag.xref
         perl ./pca -l -X ${TMP} -y
         rm -f ${TMP}/patchdiag.xref
      fi
   fi    
fi
#
# Check for the blessed Samba installation
#

SAMBA_VER=`/opt/centrify/samba/bin/smbstatus --version`
echo "Found Samba ${SAMBA_VER} "
 
/opt/centrify/samba/bin/smbstatus --version | grep -i "cdc" &> /dev/null
 
retcode=$?

if [ $retcode  -ne "0" ]; then
     $ECHO "A Centrify supported Samba does not appeared to be installed."
     QUESTION="Do you want to continue? "; DEFAULT="N"; do_askyorn
     if [ ${ANSWER} != "Y" ]; then do_quit; fi
fi


#
# Set path to Samba
#
find_samba

#
# Check if there's other version of Samba installed in the system
#
find_conflicting_samba

SMBD=${CDC_SAMBA_ROOT}/sbin/smbd
NMBD=${CDC_SAMBA_ROOT}/sbin/nmbd
WINBINDD=${CDC_SAMBA_ROOT}/sbin/winbindd
PATH=${CDC_SAMBA_ROOT}/bin:${CDC_SAMBA_ROOT}/sbin:/usr/sbin:/sbin:$PATH
export PATH

#
# Machine Joined ? 
${ECHO}
${ECHO} "Checking if machine joined to Domain ...."
ORG_CDC_DOMAIN=`$ADINFO --domain 2>/dev/null`
CDC_DOMAIN=${ORG_CDC_DOMAIN}
if [ "X${ORG_CDC_DOMAIN}" = "X" ] ; then
    $ECHO "Not joined to any domain. Make sure you enter the correct"
    $ECHO "domain and zone information in the next steps."
    $ECHO
    CDC_ZONE=default
    JOIN_CDC_DOMAIN="Y"
else
    ORG_CDC_ZONE=`$ADINFO --zone 2>/dev/null`	        
    CDC_ZONE=${ORG_CDC_ZONE}
    ${ECHO} "Joined to Domain: ${ORG_CDC_DOMAIN}"
    ${ECHO} "          Zone:   ${CDC_ZONE}"
    ${ECHO}
    QUESTION="Do you want to leave and join to another domain?"; DEFAULT="N"; do_askyorn
    JOIN_CDC_DOMAIN=${ANSWER}
    ${ECHO}
fi

if [ "${JOIN_CDC_DOMAIN}" = "Y" ]; then
    get_join_info
fi

# Looking for smb.conf to update and use.
find_smb_conf
retcode=$?
if [ $retcode  -eq "0" ]; then 
    # See if we should use existing smb.conf or update the settings in existing smb.conf with recommended settings.
    QUESTION="Do you wish to update existing smb.conf settings with recommended settings?"; DEFAULT="Y"; do_askyorn
    if [ ${ANSWER} = "N" ]; then  
       check_smbconf "${CDC_SMBCONF}" 
       retcode=$? 
       if [ $retcode  -ne "0" ]; then 
          QUESTION="Do you wish to update existing smb.conf settings with recommended settings?"; DEFAULT="Y"; do_askyorn 
          if [ ${ANSWER} = "N" ]; then 
              do_quit 
          fi 
       else 
          SMB_OPTIONS="${SMB_OPTIONS} -e" 
          SMB_CONF_MSG="Your existing SMB.CONF file will be used"     
       fi 
    fi
else 
    $ECHO "Smb.conf doesn't exist,we will generate this file with recommended settings."
fi


# Kill SMB if it is running, that may be not the Centrify version.
if [ "$OS" = "AIX" ]; then
    stopsrc -g samba
    sleep 3
fi
pkill smbd > /dev/null 2>&1
pkill nmbd > /dev/null 2>&1
pkill winbind > /dev/null 2>&1

sleep 3

# remove old state files
mv /var/lib/samba/locks/winbindd_idmap.tdb /var/lib/samba/locks/winbindd_idmap.tdb.$$ > /dev/null 2>&1
mv /usr/local/samba/var/locks/winbindd_idmap.tdb /usr/local/samba/var/locks/winbindd_idmap.tdb.$$ > /dev/null 2>&1
rm -f /etc/samba/private/secret*
rm -f /etc/samba/secret*
rm -f /usr/local/samba/private/secret*
rm -f /var/lib/samba/locks/*tdb
rm -f /var/lib/samba/locks/*dat
rm -f /usr/local/samba/var/locks/*tdb
rm -f /usr/local/samba/var/locks/*dat
rm -f /var/cache/samba/*tdb
rm -f /var/cache/samba/*dat
rm -rf /var/log/samba/*
rm -f $RC_SCRIPT_DIR/rc*.d/*smb
rm -f $RC_SCRIPT_DIR/rc*.d/*nmb
rm -f $RC_SCRIPT_DIR/rc*.d/*winbind
mv /var/lib/samba/locks/winbindd_idmap.tdb.$$ /var/lib/samba/locks/winbindd_idmap.tdb > /dev/null 2>&1
mv /usr/local/samba/var/locks/winbindd_idmap.tdb.$$ /usr/local/samba/var/locks/winbindd_idmap.tdb > /dev/null 2>&1

if [ "${JOIN_CDC_DOMAIN}" = "Y" ]; then
    do_join
else
    ${ECHO}
    QUESTION="Run adsamba in verbose mode"; DEFAULT="N"; do_askyorn
    if [ "${ANSWER}" = "Y" ]; then
        SMB_OPTIONS="${SMB_OPTIONS} -V"
    fi

    # Startup adclient if adclient is not already running 
    /usr/share/centrifydc/bin/centrifydc status | grep "Centrify DirectControl is not running" &> /dev/null
    if [ $? -eq 0 ]; then
        /usr/share/centrifydc/bin/centrifydc start
    fi
    # wait for a while until adclient completely startup
    for i in 1 2 3; do
        STATE=`$ADINFO -m` 
        if [ "${STATE}" = "connected" -o "${STATE}" = "disconnected" ]; then
            break;
        fi
        sleep 3
    done
fi

$ECHO
$ECHO "Setting up Samba and DirectControl. Please wait ..."

#Integrate Samba with CDC 
$ECHO "${ADSAMBA} ${SMB_OPTIONS} -b ${CDC_SAMBA_ROOT}"
${ADSAMBA} ${SMB_OPTIONS} -b "${CDC_SAMBA_ROOT}"
retcode=$?
if [ $retcode  -ne "0" ]; then
    do_quit
fi

sleep 10
if [ "$OS" = "AIX" ]; then
    startsrc -g samba
else
    ${INIT_SCRIPT_DIR}/centrifydc-samba start
fi
sleep 3

$ECHO
$ECHO "Current DirectControl configuration"
$ECHO "==================================="
adinfo

$ECHO
$ECHO "Current Samba configuration"
$ECHO "==========================="
net ads info

$ECHO
$ECHO "Press ENTER to continue ... "
read i

$ECHO "Notes:"
$ECHO "  If you need to join other domain, please re-run this script and enter a different domain name !"
$ECHO
$ECHO Done.
exit 0
