#!/bin/sh /usr/share/centrifydc/perl/run
 
# executable map file for automount
# it is passed the mount point name and returns the map entry for it
# or returns nothing if it cannot find an entry
# the map data comes from AD NIS map data
# the map name is derived from the executable name, so the correct thing to do is symlink 
# from /usr/share/centrifydc/etc/adauto.pl to /etc/auto.mymap
# note that this script must run as root (or at least somebody with read rights to the krb5.keytab)
# because it uses our -m option on the ldapsearch command
# our ldap package must be installed
 
use strict;
#use MIME::Base64;
use File::Spec;
 

# the map name to consult
my ($a,$b, $map) = File::Spec->splitpath($0); # i know there is a syntax to throw a and b away but I have forgotten it
my $zone_dn = "cn=$map,cn=nismaps,";
 
my $zone_base = `cat /var/centrifydc/kset.zonename`;
chomp $zone_base;
$zone_dn .= $zone_base;
 
#ldap search 
# search filter need to exclude the nismaps entries deleted in CDC console, which have prefix of a hex value of 0x1 to CN.
# description contains the map key
# admindescription contains the value
# -tt causes the output to a file 
my $ldap = `/usr/share/centrifydc/bin/ldapsearch -Q -tt -r -H "LDAP://" -b "$zone_dn" -m -LLL "(&(description=${ARGV[0]})(!(cn=\01\*)))" adminDescription`;
if ($? != 0 || $ldap eq "") {
# not found, now see if we can locate the catchall entry 
    $ldap = `/usr/share/centrifydc/bin/ldapsearch -Q -tt -r -H "LDAP://" -b "$zone_dn" -m -LLL "(&(description=\\*)(!(cn=\01\*)))" adminDescription`;
    exit if ($? != 0 || $ldap eq ""); # map not found or entry not found
}
 
my @lines = split(/\n/, $ldap);
 
# each line represent a file containing result  
foreach my $line (@lines)
{
    if($line !~ m/^ *$/)        #non blank line
    {      
        my $i = index($line, ":< file://");
        if ($i > 0)
        {   
            my $attr = substr($line, 0, $i);
            my $file = substr($line, $i + 10);
            my $value = `cat $file`;
            system "rm -f $file";
            if($attr eq "adminDescription")
            {
                print $value, "\n";
                exit 0;
            }
        }
    }
}
