#!/bin/sh /usr/share/centrifydc/perl/run
#
##############################################################################
#
# Copyright (C) 2008 Centrify Corporation.
#
# machine/user mapper script to do adreload, adflush, restart adclient, etc.
#
# Several mapper scripts need to do adreload/adflush if configuration changed.
# To prevent multiple adreload/adflush, these mapper scripts will not run
# adreload/adflush directly. Instead, they'll create:
#       /var/centrifydc/reg/do_adreload
#       /var/centrifydc/reg/do_adflush
#       /var/centrifydc/reg/do_restart_adclient
# and this mapper script will do the actual reload/flush if these files exist.
#
#
# Parameters: <map|unmap> [username] mode
#   map|unmap   action to take
#   username    username (not used)
#   mode        mode (not used)
#
# Exit value:
#   0   Normal
#   1   Error
#   2   usage
#
##############################################################################

use strict;

use lib '/usr/share/centrifydc/perl';

use CentrifyDC::GP::Args;
use CentrifyDC::GP::General qw(:debug RunCommand);

my $DO_ADRELOAD         = '/var/centrifydc/reg/do_adreload';
my $DO_ADFLUSH          = '/var/centrifydc/reg/do_adflush';
my $DO_RESTART_ADCLIENT = '/var/centrifydc/reg/do_restart_adclient';
my $DO_DSRELOAD         = '/var/centrifydc/reg/do_dsreload';
my $DO_DSFLUSH          = '/var/centrifydc/reg/do_dsflush';
my $DO_SARESTART        = '/var/centrifydc/reg/do_sarestart';

my $CMD_ADRELOAD         = '/usr/sbin/adreload';
# The flush just expires the entire cache, this way we will not lose the users cached creds.
# Losing the users creds is BAD in a laptop system since they are in disconnected mode often.
my $CMD_ADFLUSH          = '/usr/sbin/adflush -e';
my $CMD_RESTART_ADCLIENT = '/usr/share/centrifydc/bin/centrifydc restart';
# reload should be sufficient, but is not (yet)
my $CMD_DSRELOAD         = '/usr/share/centrifydc/bin/dsconfig reload';
my $CMD_DSFLUSH          = '/usr/share/centrifydc/bin/dsconfig flush';



# >>> MAIN >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

my $args = CentrifyDC::GP::Args->new();

$args->isMap() or exit(0);

if (-e $DO_RESTART_ADCLIENT)
{
    DEBUG_OUT("Restart adclient");
    my $rc = RunCommand("$CMD_RESTART_ADCLIENT", '/tmp/do_restart_adclient');
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot restart adclient");
    }
    unlink($DO_RESTART_ADCLIENT);
    unlink($DO_ADRELOAD);
}

if (-e $DO_ADRELOAD)
{
    DEBUG_OUT("Do adreload");
    my $rc = RunCommand("$CMD_ADRELOAD");
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do adreload");
    }
    unlink($DO_ADRELOAD);
}

if (-e $DO_ADFLUSH)
{
    DEBUG_OUT("Do adflush");
    my $rc = RunCommand("$CMD_ADFLUSH");
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do adflush");
    }
    unlink($DO_ADFLUSH);
}

if (-e $DO_DSRELOAD)
{
    DEBUG_OUT("Do dsreload");
    my $rc = RunCommand("$CMD_DSRELOAD");
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do dsreload");
    }
    unlink($DO_DSRELOAD);
}

if (-e $DO_DSFLUSH)
{
    DEBUG_OUT("Do dsflush");
    my $rc = RunCommand("$CMD_DSFLUSH");
    if (!defined($rc) or $rc ne '0')
    {
        ERROR_OUT("Cannot do dsflush");
    }
    unlink($DO_DSFLUSH);
}

if (-e $DO_SARESTART)
{
    DEBUG_OUT("Do sarestart (not implemented)");

    unlink($DO_SARESTART);
}

